﻿using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;
using RootPro.RootProCAD.UI.Controls;
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;

namespace RCAddInAlignTextsInFrameCS
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        private string DockingBarUniqueName = "RCAddInAlignTextsInFrame";

        public SettingsData settingsData;

        private void AppAddIn_Startup(object sender, EventArgs e)
        {
            // 多言語リソース DLL の読み込み対応
            AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveAssembly);

            // ウィンドウタイトル
            string dockingBarName = Properties.Resources.DockingBarName;

            // 初回起動か
            bool isInitialStartup = false;

            CustomDockingBar dockingBar = DockingBars.Find(DockingBarUniqueName);
            if(dockingBar == null)
            {
                isInitialStartup = true;

                // システムの解像度を取得
                var g = System.Drawing.Graphics.FromHwnd(IntPtr.Zero);
                float dpiScale = g.DpiX / 96.0f;

                // ドッキングウィンドウの初期サイズ
                int defualtWindowSize = (int)(300.0f * dpiScale);

                // ドッキングウィンドウを追加
                // 第 5 引数にtrue を指定すると RootPro CAD の画面の配色がコントロールに適用されます。
                // 第 6 引数にtrue を指定すると RootPro CAD の画面のフォントとフォントサイズがコントロールに適用されます。
                dockingBar = DockingBars.Add(DockingBarUniqueName, dockingBarName, defualtWindowSize, defualtWindowSize, true, true);
            }

            // ドッキングウィンドウのタイトルを更新
            if(dockingBar.Text != dockingBarName)
                dockingBar.Text = dockingBarName;

            // XMLファイルから設定を読み込む
            if(!LoadSettingsData())
                settingsData = new SettingsData();// 読み込めなかった場合は初期値

            // ドッキングウィンドウの画面をセット
            dockingBar.UserCtrl = new DockingBarUserControl(this);

            // 初回起動時はウィンドウサイズを調整する
            if(isInitialStartup)
                dockingBar.AdjustFloatingBar();

            // アドインメニューにマクロコマンドを追加する
            CommandManager.AddMacroCommand(DockingBarUniqueName, dockingBarName, MacroCommand);
        }

        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            // XMLファイルに設定を保存する
            SaveSettingsData();

            // アドインメニューからマクロコマンドを削除する
            CommandManager.RemoveMacroCommand(MacroCommand);
        }

        private void MacroCommand()
        {
            var dockingBar = DockingBars.Find(DockingBarUniqueName);
            if(dockingBar != null)
                // ドッキングウィンドウをアクティブにする
                dockingBar.Activate();
        }

        /// <summary>
        /// 外部フォルダのアセンブリを動的にロード
        /// 多言語対応リソースの DLL をロード
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        private Assembly ResolveAssembly(object sender, ResolveEventArgs args)
        {
            AssemblyName assemblyName = new AssemblyName(args.Name);

            foreach(Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                if(AssemblyName.ReferenceMatchesDefinition(assemblyName, assembly.GetName()))
                    return assembly;
            }

            if(assemblyName.Name.EndsWith(".resources"))
            {
                if(assemblyName.CultureName.Equals(Thread.CurrentThread.CurrentUICulture.Name, StringComparison.OrdinalIgnoreCase))
                {
                    string dllPath = Path.GetDirectoryName(this.AddInFullPath);
                    dllPath = Path.Combine(dllPath, assemblyName.CultureName);
                    dllPath = Path.Combine(dllPath, assemblyName.Name + ".dll");

                    if(File.Exists(dllPath))
                    {
                        try
                        {
                            return Assembly.LoadFrom(dllPath);
                        }
                        catch
                        {
                        }
                    }
                }
            }

            return null;
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion
    }
}
