﻿using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;

namespace RCAddInAlignTextsInFrameCS
{
    public partial class DockingBarUserControl : UserControl
    {
        AppAddIn AppAddIn;

        public DockingBarUserControl(AppAddIn app)
        {
            this.AppAddIn = app;
            InitializeComponent();

            rcAlignmentComboBox1.SelectedIndex = (int)app.settingsData.selectedAlignment;
            UpdateOffsetEdit();
        }

        /// <summary>
        /// X 方向の余白のテキストボックスが変更されたときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            double value;
            Alignment alignment = AppAddIn.settingsData.selectedAlignment;
            if(textBox1.Text.Length == 0)
                AppAddIn.settingsData.SetOffsetX(alignment, 0);
            else if(textBox1.Text.Length == 1 && textBox1.Text.IndexOfAny(new char[] { '-', '+' }) == 0)
            { }
            else if(double.TryParse(textBox1.Text, out value))
                AppAddIn.settingsData.SetOffsetX(alignment, value);
            else
            {
                textBox1.Text = AppAddIn.settingsData.GetOffsetX(alignment).ToString();
                textBox1.SelectAll();
            }
        }

        /// <summary>
        /// Y 方向の余白のテキストボックスが変更されたときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private void textBox2_TextChanged(object sender, EventArgs e)
        {
            double value;
            Alignment alignment = AppAddIn.settingsData.selectedAlignment;
            if(textBox2.Text.Length == 0)
                AppAddIn.settingsData.SetOffsetY(alignment, 0);
            else if(textBox2.Text.Length == 1 && textBox2.Text.IndexOfAny(new char[] { '-', '+' }) == 0)
            { }
            else if(double.TryParse(textBox2.Text, out value))
                AppAddIn.settingsData.SetOffsetY(alignment, value);
            else
            {
                textBox2.Text = AppAddIn.settingsData.GetOffsetY(alignment).ToString();
                textBox2.SelectAll();
            }
        }

        /// <summary>
        /// 配置基準点のコンボボックスが変更されたときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private void rcAlignmentComboBox1_SelectedValueChanged(object sender, EventArgs e)
        {
            UpdateOffsetEdit();
        }

        /// <summary>
        /// 余白テキストボックスの更新処理
        /// </summary>
        /// <returns></returns>
        private void UpdateOffsetEdit()
        {
            Alignment alignment = (Alignment)rcAlignmentComboBox1.SelectedIndex;
            AppAddIn.settingsData.selectedAlignment = alignment;
            textBox1.Text = AppAddIn.settingsData.GetOffsetX(alignment).ToString();
            textBox2.Text = AppAddIn.settingsData.GetOffsetY(alignment).ToString();
        }

        /// <summary>
        /// 設定の読み込みが選択されたときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private void LoadSettingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //ファイルダイアログの作成
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.InitialDirectory = @"";
            fileDialog.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
            fileDialog.FilterIndex = 1;
            fileDialog.RestoreDirectory = true;

            if(fileDialog.ShowDialog() == DialogResult.OK)
            {
                // XML ファイルから設定を読み込む
                AppAddIn.LoadSettingsData(fileDialog.FileName);
                UpdateOffsetEdit();
            }
        }

        /// <summary>
        /// 設定の保存が選択されたときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private void SaveSettingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //ファイルダイアログの作成
            SaveFileDialog fileDialog = new SaveFileDialog();
            fileDialog.InitialDirectory = @"";
            fileDialog.FileName = "settins.xml";
            fileDialog.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
            fileDialog.Title = "保存"/*Properties.Resources.SaveFileDialogTitle*/;
            fileDialog.FilterIndex = 1;
            fileDialog.RestoreDirectory = true;

            if(fileDialog.ShowDialog() == DialogResult.OK)
            {
                // XML ファイルに設定を保存する
                AppAddIn.SaveSettingsData(fileDialog.FileName);
            }
        }

        /// <summary>
        /// 実行ボタンをクリックしたときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private void button1_Click(object sender, EventArgs e)
        {
            var doc = AppAddIn.ActiveDocument;
            if(doc != null)
            {
                // 選択図形取得
                var selectedShapes = doc.SelectionManager.SelectedShapes;
                int aligmentCount = 0;

                // Start UNDO process
                doc.UndoManager.BeginUndoUnit();

                foreach(var selectedShape in selectedShapes)
                {
                    // 文字図形のみ
                    if(!(selectedShape.Shape is TextShape))
                        continue;

                    // Text shape
                    TextShape text = (TextShape)selectedShape.Shape;

                    // テキストを囲む矩形領域を取得
                    var textBoundingBox = text.GetBoundingBox();

                    // テキストの中心座標を取得する
                    var localPoint = AppAddIn.Geometry.GetMiddlePoint(textBoundingBox[0], textBoundingBox[1]);

                    // 選択図形のレイアウトを取得
                    Layout layout = selectedShape.Layout;
                    if(doc.CurrentLayout != layout)
                    {
                        // ワールド座標系に変換する
                        Point2d worldPoint = layout.TransformPointLocalToWorld(localPoint);
                        localPoint = doc.CurrentLayout.TransformPointWorldToLocal(worldPoint);
                    }

                    // 現在の部分図ビューを取得する
                    DrawingView drawView = (DrawingView)AppAddIn.ActiveTabView;


                    // 文字の中心点座標でリージョンを検索する
                    RegionSearchResult result = doc.RegionSearcher.Find(drawView, localPoint);

                    // リージョン検索が成功した場合
                    if(result.Succeeded == true)
                    {
                        // ドッキングウィンドウで選択している基準点
                        Alignment selectedAlignment = AppAddIn.settingsData.selectedAlignment;

                        double offsetX = AppAddIn.settingsData.GetOffsetX(selectedAlignment);
                        double offsetY = AppAddIn.settingsData.GetOffsetY(selectedAlignment);

                        // 文字図形の配置位置を取得する
                        Point2d locateRegionPoint = GetLocateRegionPoint(result.RegionShape, selectedAlignment, offsetX, offsetY);

                        if(doc.CurrentLayout != layout)
                        {
                            // ワールド座標系に変換する
                            Point2d worldPoint = doc.CurrentLayout.TransformPointLocalToWorld(locateRegionPoint);
                            locateRegionPoint = layout.TransformPointWorldToLocal(worldPoint);
                        }

                        // 文字の位置を調整
                        AdjustTextAlignment(text, selectedAlignment, locateRegionPoint);

                        aligmentCount++;

                    }
                }
                // Terminate UNDO process
                doc.UndoManager.EndUndoUnit();


                if(aligmentCount == 0)
                {
                    MessageBox.Show(Properties.Resources.MessageSelectTextShape);
                }
            }
        }

        /// <summary>
        /// 文字図形の位置を調整
        /// </summary>
        /// <param name="text"></param>
        /// <param name="Alignment"></param>
        /// <param name="point"></param>
        /// <returns></returns>
        private void AdjustTextAlignment(TextShape text, Alignment alignment, Point2d point)
        {
            // 角度は考慮せず文字のレイアウトの幅、高さを参照して位置を調整する
            double textWidth = text.LayoutWidth;
            double textHeight = text.LayoutHeight;

            // 補正係数を定義
            double[,] correctionFactors = new double[3, 3]{
                { 0.0, -0.5, -1.0}, { 0.5, 0.0, -0.5 }, { 1.0, 0.5, 0.0}
            };

            // 文字のアライメント
            Alignment textAlignment = text.Alignment;

            // X 座標の補正値を計算
            int coffX1 = (int)textAlignment % 3;
            int coffX2 = (int)alignment % 3;
            double correctValueX = textWidth * correctionFactors[coffX1, coffX2];

            // Y 座標の補正値を計算
            int coffY1 = (int)textAlignment / 3;
            int coffY2 = (int)alignment / 3;
            double correctValueY = textHeight * correctionFactors[coffY1, coffY2];

            point.X += correctValueX;
            point.Y += correctValueY;

            text.Point = point;
        }

        /// <summary>
        /// リージョン内の指定したアライメントの位置を取得する
        /// </summary>
        /// <param name="regionShape"></param>
        /// <param name="alignment"></param>
        /// <param name="offsetX"></param>
        /// <param name="offsetY"></param>
        /// <returns></returns>
        private Point2d GetLocateRegionPoint(RegionShape regionShape, Alignment alignment, double offsetX, double offsetY)
        {
            // リージョンを囲む矩形領域を取得
            var boundingBox = regionShape.GetBoundingBox();

            // リージョンの中心座標を取得する
            var centerPoint = AppAddIn.Geometry.GetMiddlePoint(boundingBox[0], boundingBox[1]);

            Point2d locatePoint = new Point2d();
            switch(alignment)
            {
                case Alignment.BottomLeft:
                    locatePoint.X = boundingBox[0].X + offsetX;
                    locatePoint.Y = boundingBox[0].Y + offsetY;
                    break;
                case Alignment.BottomMiddle:
                    locatePoint.X = centerPoint.X + offsetX;
                    locatePoint.Y = boundingBox[0].Y + offsetY;
                    break;
                case Alignment.BottomRight:
                    locatePoint.X = boundingBox[1].X - offsetX;
                    locatePoint.Y = boundingBox[0].Y + offsetY;
                    break;
                case Alignment.MiddleLeft:
                    locatePoint.X = boundingBox[0].X + offsetX;
                    locatePoint.Y = centerPoint.Y - offsetY;
                    break;
                case Alignment.MiddleMiddle:
                    locatePoint.X = centerPoint.X + offsetX;
                    locatePoint.Y = centerPoint.Y - offsetY;
                    break;
                case Alignment.MiddleRight:
                    locatePoint.X = boundingBox[1].X - offsetX;
                    locatePoint.Y = centerPoint.Y - offsetY;
                    break;
                case Alignment.TopLeft:
                    locatePoint.X = boundingBox[0].X + offsetX;
                    locatePoint.Y = boundingBox[1].Y - offsetY;
                    break;
                case Alignment.TopMiddle:
                    locatePoint.X = centerPoint.X + offsetX;
                    locatePoint.Y = boundingBox[1].Y - offsetY;
                    break;
                case Alignment.TopRight:
                    locatePoint.X = boundingBox[1].X - offsetX;
                    locatePoint.Y = boundingBox[1].Y - offsetY;
                    break;
            }
            return locatePoint;
        }
    }
}
