﻿using RootPro.RootProCAD;
using System;
using System.Collections.Generic;
using System.IO;

namespace RCAddInAlignTextsInFrameCS
{
    partial class AppAddIn
    {
        // SettingsData クラスの保存XMLファイル名
        public string RC_SettingsDataXmlFileName = "RCAddInAlingTextInFrameSettings.xml";

        /// <summary>
        /// オフセットデータ
        /// </summary>
        [Serializable]
        public class OffsetData
        {
            // オフセット値 X
            public double offestX;

            // オフセット値 Y
            public double offestY;

            /// <summary>
            /// コンストラクタ―
            /// </summary>
            public OffsetData()
            {
                // オフセット値 X
                offestX = 0.0;

                // オフセット値 Y
                offestY = 0.0;
            }
        }

        /// <summary>
        /// データクラス
        /// </summary>
        public class SettingsData
        {
            // 選択中の基準点
            public Alignment selectedAlignment;

            // オフセット値
            public OffsetData[] offsets;

            /// <summary>
            /// コンストラクタ―
            /// </summary>
            public SettingsData()
            {
                selectedAlignment = Alignment.MiddleMiddle;
                offsets = new OffsetData[9];

                for(int i = 0; i < 9; i++)
                    offsets[i] = new OffsetData();
            }

            /// <summary>
            /// オフセット値 X をセット 
            /// </summary>
            public void SetOffsetX(Alignment alignment, double offsetX)
            {
                offsets[(int)alignment].offestX = offsetX;
            }

            /// <summary>
            /// オフセット値 Y をセット 
            /// </summary>
            public void SetOffsetY(Alignment alignment, double offsetY)
            {
                offsets[(int)alignment].offestY = offsetY;
            }

            /// <summary>
            /// オフセット値 X を取得 
            /// </summary>
            public double GetOffsetX(Alignment alignment)
            {
                return offsets[(int)alignment].offestX;
            }

            /// <summary>
            /// オフセット値 Y を取得 
            /// </summary>
            public double GetOffsetY(Alignment alignment)
            {
                return offsets[(int)alignment].offestY;
            }
        }

        /// <summary>
        /// デフォルト設定を保存
        /// </summary>
        public void SaveSettingsData()
        {
            // 保存ファイルパス
            string addinDir = Path.GetDirectoryName(AddInFullPath);
            string filePathDefault = addinDir + "\\" + RC_SettingsDataXmlFileName;

            SaveSettingsData(filePathDefault);

        }

        /// <summary>
        /// 設定値をファイルに保存
        /// </summary>
        public void SaveSettingsData(string filePath)
        {
            System.IO.StreamWriter sw = null;

            try
            {
                // 書き込むファイルを開く（UTF-8 BOM無し）
                sw = new System.IO.StreamWriter(filePath, false, new System.Text.UTF8Encoding(false));

                // XmlSerializerオブジェクトを作成
                // CustomSettings
                System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(SettingsData));

                //シリアル化し、XMLファイルに保存する
                serializer.Serialize(sw, settingsData);

                //ファイルを閉じる
                sw.Close();
            }
            catch(System.Exception)
            {

            }
            finally
            {
                if(sw != null)
                {
                    sw.Close();
                }
            }
        }

        /// <summary>
        /// デフォルト設定を読み込む
        /// </summary>
        public bool LoadSettingsData()
        {
            // 保存ファイルパス
            string addinDir = Path.GetDirectoryName(AddInFullPath);
            string filePathDefault = addinDir + "\\" + RC_SettingsDataXmlFileName;

            return LoadSettingsData(filePathDefault);
        }


        /// <summary>
        /// 設定をファイルから読み込む
        /// </summary>
        public bool LoadSettingsData(string filePath)
        {
            settingsData = null;

            //読み込むファイルを開く
            StreamReader sr = null;
            try
            {
                sr = new StreamReader(filePath, new System.Text.UTF8Encoding(false));

                // XmlSerializerオブジェクトを作成
                // SettingsData
                System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(SettingsData));

                // XMLファイルから読み込み、逆シリアル化する
                settingsData = (SettingsData)serializer.Deserialize(sr);

                //ファイルを閉じる
                sr.Close();

                return true;
            }
            catch(System.Exception)
            {

            }
            finally
            {
                if(sr != null)
                {
                    sr.Close();
                }
            }

            return false;
        }
    }
}